@@ -204,30 +204,129 @@ def _setup_transitions(tdef, states, prev=()):
     return TransitionList(trs)
 
 
-class TransitionImplementation(object):
+class ImplementationWrapper(object):
+    """Wraps a transition implementation.
+
+    Emulates a function behaviour, but provides a few extra features.
+
+    Attributes:
+        instance (WorkflowEnabled): the instance to update
+.
+        field_name (str): the name of the field of the instance to update.
+        transition (Transition): the transition to perform
+        workflow (Workflow): the workflow to which this is related.
+
+        check (callable): optional function to call along with state checks to
+            determine whether the transition is available.
+        before (callable): optional callable to call *before* performing the
+            transition.
+        implementation (callable): the code to invoke between 'before' and the
+            state update.
+        after (callable): optional callable to call *after* changing the state
+            and logging the transition.
+    """
+
+    def __init__(self, instance, field_name, transition, workflow,
+            implementation, check=None, before=None, after=None):
+        self.instance = instance
+        self.field_name = field_name
+        self.transition = transition
+        self.workflow = workflow
+
+        self.check = check
+        self.before = before
+        self.implementation = implementation
+        self.after = after
+
+        self.__doc__ = implementation.__doc__
+
+    def _pre_transition_checks(self):
+        """Run the pre-transition checks."""
+        current_state = getattr(self.instance, self.field_name)
+        if current_state not in self.transition.source:
+            raise InvalidTransitionError(
+                "Transition '%s' isn't available from state '%s'." %
+                (self.transition.name, current_state.name))
+
+        if self.check is not None:
+            if not self.check(self.instance):
+                raise ForbiddenTransition(
+                    "Transition '%s' was forbidden by "
+                    "custom pre-transition check." % self.transition.name)
+
+    def _pre_transition(self, *args, **kwargs):
+        if self.before is not None:
+            self.before(self.instance, *args, **kwargs)
+
+    def _during_transition(self, *args, **kwargs):
+        return self.implementation(self.instance, *args, **kwargs)
+
+    def _log_transition(self, from_state, *args, **kwargs):
+        self.workflow.log_transition(self.transition, from_state, self.instance,
+            *args, **kwargs)
+
+    def _post_transition(self, result, *args, **kwargs):
+        """Performs post-transition actions."""
+        if self.after is not None:
+            self.after(self.instance, result, *args, **kwargs)
+
+    def __call__(self, *args, **kwargs):
+        """Run the transition, with all checks."""
+
+        self._pre_transition_checks()
+        # Call hooks.
+        self._pre_transition(*args, **kwargs)
+
+        result = self._during_transition(*args, **kwargs)
+
+        from_state = getattr(self.instance, self.field_name)
+        setattr(self.instance, self.field_name, self.transition.target)
+
+        # Call hooks.
+        self._log_transition(from_state, *args, **kwargs)
+        self._post_transition(result, *args, **kwargs)
+        return result
+
+    def is_available(self):
+        """Check whether this transition is available on the current object.
+
+        Returns:
+            bool
+        """
+        try:
+            self._pre_transition_checks()
+        except (InvalidTransitionError, ForbiddenTransition):
+            return False
+        return True
+
+    def __repr__(self):
+        return "<%s for '%s' on '%s': %s>" % (self.__class__.__name__,
+            self.transition.name, self.field_name, self.implementation)
+
+
+class ImplementationProperty(object):
     """Holds an implementation of a transition.
 
     This class is a 'non-data descriptor', somewhat similar to a property.
 
-    The 'implementation' callable is called with the modified object as its
-    first argument; it may raise a AbortTransition exception to cancel the
-    transition.
-
     Attributes:
-        transition (Transition): the related transition
-        field_name (str): the name of the field storing the state (which should
-            be modified when the transition is called)
-        implementation (callable): the actual function to call when performing
-            the transition.
+        field_name (str): the name of the field of the instance to update.
+        transition (Transition): the transition to perform
+        workflow (Workflow): the workflow to which this is related.
+
+        check (callable): optional function to call along with state checks to
+            determine whether the transition is available.
         before (callable): optional callable to call *before* performing the
-            transition (but after state checks). If the return value evaluates
-            to False, the transition will be silently aborted.
-        after (callable): optional callable to call *after* performing the
-            transition (once the state has changed).
+            transition.
+        implementation (callable): the code to invoke between 'before' and the
+            state update.
+        after (callable): optional callable to call *after* changing the state
+            and logging the transition.
     """
-    def __init__(self, transition, field_name, workflow, implementation, check=None, before=None, after=None):
-        self.transition = transition
+    def __init__(self, field_name, transition, workflow, implementation,
+            check=None, before=None, after=None):
         self.field_name = field_name
+        self.transition = transition
         self.workflow = workflow
         self.check = check
         self.before = before
@@ -241,63 +340,17 @@ class TransitionImplementation(object):
 
         if not isinstance(instance, BaseWorkflowEnabled):
             raise TypeError(
-                "Unable to apply transition %s to object %s, which is not "
-                "attached to a Workflow." % (self.transition, instance))
+                "Unable to apply transition '%s' to object %r, which is not "
+                "attached to a Workflow." % (self.transition.name, instance))
 
-        @functools.wraps(self.implementation)
-        def actual_implementation(*args, **kwargs):
-            return self._run_implem(instance, *args, **kwargs)
-
-        return actual_implementation
-
-    def _check_state(self, instance):
-        current_state = getattr(instance, self.field_name)
-        if current_state not in self.transition.source:
-            raise InvalidTransitionError(
-                "Transition %s isn't available from state %s." %
-                (self.transition, current_state))
-
-    def _pre_transition(self, instance, *args, **kwargs):
-        if self.check is not None:
-            if not self.check(instance):
-                raise ForbiddenTransition(
-                    "Transition %s was forbidden by "
-                    "custom pre-transition check." % self.transition.name)
-
-        if self.before is not None:
-            self.before(instance, *args, **kwargs)
-
-    def _run_implem(self, instance, *args, **kwargs):
-        """Run the transition, with all checks."""
-
-        self._check_state(instance)
-        # Call hooks.
-        self._pre_transition(instance, *args, **kwargs)
-
-        res = self._during_transition(instance, *args, **kwargs)
-
-        from_state = getattr(instance, self.field_name)
-        setattr(instance, self.field_name, self.transition.target)
-
-        # Call hooks.
-        self._log_transition(instance, from_state, *args, **kwargs)
-        self._post_transition(instance, res, *args, **kwargs)
-        return res
-
-    def _during_transition(self, instance, *args, **kwargs):
-        return self.implementation(instance, *args, **kwargs)
-
-    def _log_transition(self, instance, from_state, *args, **kwargs):
-        self.workflow.log_transition(self.transition, from_state, instance,
-            *args, **kwargs)
-
-    def _post_transition(self, instance, res, *args, **kwargs):
-        """Performs post-transition actions."""
-        if self.after is not None:
-            self.after(instance, res, *args, **kwargs)
+        return self.workflow.implementation_class(instance,
+            self.field_name, self.transition, self.workflow,
+            self.implementation,
+            self.check, self.before, self.after)
 
     def __repr__(self):
-        return "<%s for %s on '%s': %s>" % (self.__class__.__name__, self.transition, self.field_name, self.implementation)
+        return "<%s for '%s' on '%s': %s>" % (self.__class__.__name__,
+            self.transition.name, self.field_name, self.implementation)
 
 
 class TransitionWrapper(object):
@@ -345,118 +398,111 @@ class ImplementationList(object):
 
     Attributes:
         state_field (str): the name of the field holding the state of objects.
-        _implems (dict(str => TransitionImplementation)): maps an attribute name
-            to the associated transition implementation.
-        _transitions (TransitionList): list of expected transitions.
-        _transitions_mapping (dict(str => str)): maps a transition name to the
+        implementations (dict(str => ImplementationProperty)): maps a transition
+            name to the associated implementation.
+        workflow (Workflow): the related workflow
+        transitions_at (dict(str => str)): maps a transition name to the
             name of the attribute holding the related transition.
     """
 
     def __init__(self, state_field, workflow):
         self.state_field = state_field
-        self._implems = {}
-        self._transitions_mapping = {}
-        self._workflow = workflow
+        self.workflow = workflow
+        self.implementations = {}
+        self.transitions_at = {}
 
-    def _get_implem(self, transition_name):
-        return self._implems[self._transitions_mapping[transition_name]]
+    def add_implem(self, transition, attribute, function, **kwargs):
+        """Add an implementation.
+
+        Args:
+            transition (Transition): the transition for which the implementation
+                is added
+            attribute (str): the name of the attribute where the implementation
+                will be available
+            function (callable): the actual implementation function
+            **kwargs: extra arguments for the related ImplementationProperty.
+        """
+        implem = ImplementationProperty(
+            field_name=self.state_field,
+            transition=transition,
+            workflow=self.workflow,
+            implementation=function,
+            **kwargs)
+        self.implementations[transition.name] = implem
+        self.transitions_at[transition.name] = attribute
+
+    def should_collect(self, value):
+        """Decide whether a given value should be collected."""
+        return (
+            # decorated with @transition
+            isinstance(value, TransitionWrapper)
+            # Relates to a compatible transition
+            and value.trname in self.workflow.transitions
+            # Either not bound to a state field or bound to the current one
+            and (not value.field or value.field == self.state_field))
 
     def collect(self, attrs):
         """Collect the implementations from a given attributes dict."""
 
-        def add_implem(transition, attr_name, function, check=None, before=None, after=None):
-            """Helper for adding a an implementation to the list."""
-            implem = self._workflow.implementation_class(
-                transition, self.state_field, self._workflow, function,
-                check=check, before=before, after=after)
-            self._implems[attr_name] = implem
-            self._transitions_mapping[transition.name] = attr_name
-
         for name, value in attrs.iteritems():
-            if isinstance(value, TransitionWrapper):
-                if value.trname in self._workflow.transitions:
-                    if value.field and value.field != self.state_field:
-                        # Applying for another workflow
-                        continue
+            if self.should_collect(value):
+                transition = self.workflow.transitions[value.trname]
 
-                    transition = self._workflow.transitions[value.trname]
-
-                    if value.trname in self._transitions_mapping:
-                        # Some other function used @transition(value.trname)
-                        other_implem = self._get_implem(value.trname)
-                        raise ValueError(
-                            "Error for attribute %s: it defines implementation "
-                            "%s for transition %s, which is already implemented "
-                            "as %s." % (name, value, transition,
-                                self._get_implem(value.trname)))
-
-                    add_implem(transition, name, value.func,
-                        check=value.check, before=value.before, after=value.after)
-
-    def _assert_may_override(self, implem, other, attrname):
-        if isinstance(other, TransitionImplementation):
-            # Another TransitionImplementation, but not the same transition/workflow
-            if other.transition != implem.transition or other.field_name != implem.field_name:
-                raise ValueError(
-                    "Can't override transition implementation %s=%s with %s" %
-                    (attrname, other, implem))
-        elif isinstance(other, TransitionWrapper):
-            # A TransitionWrapper, but not the same Transition
-            if other.trname != implem.transition.name:
-                raise ValueError(
-                    "Can't override transition implementation %s=%s with %s" %
-                    (attrname, other, implem))
-        elif other != implem.implementation:
-            # Anything else, and not what this TransitionImplementation wrap.
-            raise ValueError(
-                "Can't override attribute %s=%s with %s" %
-                (attrname, other, implem))
-
-    @classmethod
-    def _add_implem(cls, attrs, attrname, implem):
-        attrs[attrname] = implem
-
-    def update_attrs(self, attrs):
-        for transition in self._workflow.transitions:
-            # Where to put the implementation
-            attrname = transition.name
-
-            if transition.name in self._transitions_mapping:
-                # We had a custom implementation
-                attrname = self._transitions_mapping[transition.name]
-                implem = self._implems[attrname]
-                if implem.transition != transition:
-                    # Should never happen...
+                if value.trname in self.implementations:
+                    # We already have an implementation registered.
+                    other_implem_at = self.transitions_at[value.trname]
                     raise ValueError(
-                        "Error for transition %s: implementation has been "
-                        "overridden by %s." % (transition, implem))
-            elif attrname in attrs:
-                # No custom implementation, but name conflict.
-                raise ValueError(
-                    "Error for transition %s: no implementation is defined, "
-                    "and the related attribute lacks a @transition "
-                    "decorator: %s" % (transition, attrs[attrname]))
+                        "Error for attribute %s: it defines implementation "
+                        "%s for transition %s, which is already implemented "
+                        "at %s." % (name, value, transition, other_implem_at))
 
-            else:
-                # No custom implementation, no name conflict.
-                # Create a 'noop' transition
-                implem = self._workflow.implementation_class(
-                    transition, self.state_field, self._workflow, noop)
+                self.add_implem(transition, name, value.func,
+                    check=value.check, before=value.before, after=value.after)
+
+    def add_missing_implementations(self):
+        for transition in self.workflow.transitions:
+            if transition.name not in self.implementations:
+                self.add_implem(transition, transition.name, noop)
+
+    def _may_override(self, implem, other):
+        """Checks whether an ImplementationProperty may override an attribute."""
+        if isinstance(other, ImplementationProperty):
+            # Overriding another custom implementation for the same transition
+            # and field
+            return (other.transition == implem.transition
+                and other.field_name == self.state_field)
+
+        elif isinstance(other, TransitionWrapper):
+            # Overriding the definition that led to adding the current
+            # ImplementationProperty.
+            return (other.trname == implem.transition.name
+                and (not other.field or other.field == self.state_field)
+                and other.func == implem.implementation)
+
+        return False
+
+    def fill_attrs(self, attrs):
+        """Update the 'attrs' dict with generated ImplementationProperty."""
+        for trname, attrname in self.transitions_at.items():
+
+            implem = self.implementations[trname]
 
             if attrname in attrs:
-                self._assert_may_override(implem, attrs[attrname], attrname)
+                conflicting = attrs[attrname]
+                if not self._may_override(implem, conflicting):
+                    raise ValueError(
+                        "Can't override transition implementation %s=%r with %r" %
+                        (attrname, conflicting, implem))
 
-            self._add_implem(attrs, attrname, implem)
+            attrs[attrname] = implem
         return attrs
 
-    def __getitem__(self, trname):
-        attrname = self._transitions_mapping[trname]
-        return self._implems[attrname]
-
-    def __contains__(self, trname):
-        if not trname in self._transitions_mapping:
-            return False
-        return self._transitions_mapping[trname] in self._implems
+    def transform(self, attrs, add_missing=True):
+        """Perform all actions on a given attribute dict."""
+        self.collect(attrs)
+        if add_missing:
+            self.add_missing_implementations()
+        self.fill_attrs(attrs)
 
     def __repr__(self):
         return '<%s: %r>' % (self.__class__.__name__, self._implems.values())
@@ -493,16 +539,16 @@ class Workflow(object):
         states (StateList): list of states of this Workflow
         transitions (TransitionList): list of Transitions of this Workflow
         initial_state (State): initial state for the Workflow
-        implementation_class (TransitionImplementation subclass): class to use
+        implementation_class (ImplementationWrapper subclass): class to use
             for transition implementation wrapping.
 
-    For each transition, a TransitionImplementation with the same name (unless
+    For each transition, a ImplementationWrapper with the same name (unless
     another name has been specified through the use of the @transition
     decorator) is provided to perform the specified transition.
     """
     __metaclass__ = WorkflowMeta
 
-    implementation_class = TransitionImplementation
+    implementation_class = ImplementationWrapper
 
     def log_transition(self, transition, from_state, instance, *args, **kwargs):
         """Log a transition.
@@ -622,6 +668,9 @@ class WorkflowEnabledMeta(type):
         """Attach a workflow to the attribute list (create a StateProperty)."""
         attrs[field_name] = StateProperty(state_field.workflow, field_name)
 
+        implems = ImplementationList(field_name, state_field.workflow)
+        implems.transform(attrs)
+
     @classmethod
     def _find_workflows(mcs, attrs):
         """Finds all occurrences of a workflow in the attributes definitions.
@@ -631,22 +680,17 @@ class WorkflowEnabledMeta(type):
                 describing the related Workflow.
         """
         workflows = {}
-        for k, v in attrs.items():
-            if isinstance(v, Workflow):
-                workflows[k] = StateField(v)
+        for attribute, value in attrs.items():
+            if isinstance(value, Workflow):
+                workflows[attribute] = StateField(value)
         return workflows
 
     def __new__(mcs, name, bases, attrs):
         workflows = mcs._find_workflows(attrs)
-        for field_name, state_field in workflows.iteritems():
+        for field_name, state_field in workflows.items():
             mcs._add_workflow(field_name, state_field, attrs)
 
-            implems = ImplementationList(field_name, state_field.workflow)
-            implems.collect(attrs)
-            implems.update_attrs(attrs)
-
-        attrs['_workflows'] = dict((field_name, state_field.workflow)
-            for field_name, state_field in workflows.items())
+        attrs['_workflows'] = workflows
         return super(WorkflowEnabledMeta, mcs).__new__(mcs, name, bases, attrs)
 
 
@@ -654,7 +698,7 @@ class BaseWorkflowEnabled(object):
     """Base class for all objects using a workflow.
 
     Attributes:
-        _workflows (dict(str, Workflow)): Maps the name of a 'state_field' to
+        workflows (dict(str, StateField)): Maps the name of a 'state_field' to
             the related Workflow
     """
 
